
# q-expansion of newform 666.2.a.h, downloaded from the LMFDB on 09 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 666
weight = 2
poly_data = [-4, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1, 0], [0, 0], [-2, 0], [1, -1], [-1, 1], [-1, 3], [-1, -1], [-1, -1], [-1, -3], [-4, 2], [-4, -2], [-1, 0], [-6, -2], [-4, 2], [0, 4], [-1, 1], [-6, 2], [-2, -4], [-8, 4], [8, -4], [1, 1], [-6, 0], [3, -7], [-5, 7], [8, -2], [6, 2], [-10, 0], [9, -1], [1, 1], [2, 4], [3, 1], [-10, -2], [16, -4], [-12, 0], [2, -2], [11, -3], [10, -8], [-3, -7], [9, 3], [1, -1], [-2, -2], [-6, 8], [-5, 1], [16, -6], [1, -5], [8, 2], [-4, 0], [8, 8], [-10, 6], [12, 6], [16, 0], [-8, 0], [2, -4], [-2, -2], [-21, -1], [-24, 4], [5, -9], [-16, 4], [25, 1], [-5, 7], [-19, 5], [-5, 13], [-2, -6], [-20, -4], [-8, 2], [2, 2], [-8, -6], [13, -7], [14, -10], [14, -8], [-26, 0], [8, 8], [17, 3], [12, 6], [4, 4], [1, -5], [14, -12], [6, -8], [-9, -1], [10, 4], [-31, 3], [10, 4], [9, 11], [23, -1], [10, -8], [-12, 12], [-22, -4], [-20, 10], [-14, 16], [4, -6], [-6, 10], [-19, -9], [8, 2], [19, -7], [5, -15], [8, -16], [-25, 9], [-24, 0], [-24, -2], [-21, 7], [-15, 5], [34, 0], [14, -2], [5, 13], [-20, 12], [-4, -2], [24, -4], [-18, -6], [24, 4], [-17, 3], [0, 14], [10, -16], [18, 2], [-12, 12], [18, -12], [-14, 14], [15, -5], [3, -7], [14, -12], [8, 4], [-17, -1], [-23, 9], [25, -1], [22, -18], [-26, -2], [-36, 6], [-9, -13], [-38, 6], [-16, -6], [-12, -14], [22, 10], [-28, 16], [12, -20], [-3, -3], [6, -18], [20, -10], [9, 3], [-16, 20], [-34, 0], [-1, 15], [2, 6], [43, -3], [7, 5], [40, 4], [1, 1], [-26, -2], [39, -1], [5, -19], [-23, 9], [-42, -6], [34, 0], [4, -8], [-31, 1], [-18, -2], [5, 13], [-16, -8], [34, 4], [-36, -4], [-22, -4], [14, 14], [-10, 6], [-14, -14], [12, -6], [-20, 4], [-25, 23], [-6, 10], [30, 12], [-19, -7]]
